@@ -158,23 +158,41 @@ checkPackageSize <- function(pkg, pkgdir, size=5){
     }
 }
 
+.MAX_FILE_SIZE <- 5*10^6 ## 5MB
+
+.findLargeFiles <- function(pkgdir, data_only) {
+    stopifnot(is.logical(data_only))
+    folders <- list.dirs(pkgdir, full.names = TRUE, recursive = TRUE)
+    data_folders <- basename(folders) %in% c("data", "extdata", "data-raw")
+    if (data_only)
+        folders <- folders[data_folders]
+    else
+        folders <- folders[!data_folders]
+    files <- list.files(folders, full.names = TRUE, recursive = TRUE)
+    filesizes <- file.size(files)
+    files[filesizes > .MAX_FILE_SIZE]
+}
+
 checkIndivFileSizes <- function(pkgdir)
 {
-    pkgType <- getPkgType(pkgdir)
-    if (is.na(pkgType) ||  pkgType == "Software") {
-        maxSize <- 5*10^6 ## 5MB
-        allFiles <- list.files(pkgdir, all.files=TRUE, recursive=TRUE)
-        allFilesFullName <- file.path(pkgdir, allFiles)
-        sizes <- file.size(allFilesFullName)
-        largeFiles <- paste(allFiles[sizes > maxSize], collapse=" ")
-        if (any(sizes > maxSize)) {
-            handleWarning(
-                "The following files are over 5MB in size: ",
-                paste0("'", largeFiles, "'", collapse = " ")
-            )
-            return(TRUE)
-        }
-    }
+    largefiles <- .findLargeFiles(pkgdir, data_only = FALSE)
+    if (length(largefiles))
+        handleWarning(
+            "Package files exceed the 5MB size limit.",
+            help_text = "Files over the limit: ",
+            messages = largefiles
+        )
+}
+
+checkDataFileSizes <- function(pkgdir) {
+    largedata <- .findLargeFiles(pkgdir, data_only = TRUE)
+    if (length(largedata))
+        handleWarning(
+            "Data files exceed the 5MB size limit.",
+            help_text =
+                "Use 'ExperimentHub' or 'AnnotationHub' for the files: ",
+            messages = largedata
+        )
 }
 
 .testRbuildignore <- function(text) {
@@ -205,6 +223,15 @@ checkBiocCheckOutputFolder <- function(pkgdir, pkg_name) {
         )
 }
 
+checkInstDocFolder <- function(pkgdir, pkg_name) {
+    alldirs <- list.dirs(pkgdir, full.names = FALSE)
+    instdocfiles <- list.files(file.path(pkgdir, "inst/doc"))
+    if ("inst/doc" %in% alldirs && length(instdocfiles))
+        handleError(
+            "Remove 'inst/doc' folder from the package source"
+        )
+}
+
 checkBiocViews <- function(pkgdir)
 {
     dirty <- FALSE
@@ -525,23 +552,6 @@ checkDescriptionNamespaceConsistency <- function(pkgname, lib.loc)
     }
 }
 
-checkImportSuggestions <- function(pkgname)
-{
-    suggestions <- try(
-        suppressMessages(suppressWarnings(
-            capture.output(codetoolsBioC::writeNamespaceImports(pkgname))
-        )), silent = TRUE
-    )
-    if (inherits(suggestions, "try-error")) {
-        handleMessage("Could not get namespace suggestions.")
-    } else {
-        handleMessage("Namespace import suggestions are:")
-        handleVerbatim(suggestions, indent=4, exdent=4, width=100000)
-        handleMessage("--END of namespace import suggestions.")
-    }
-    suggestions
-}
-
 checkVignetteDir <- function(pkgdir, checkingDir)
 {
     vigdir <- file.path(pkgdir, "vignettes")
@@ -598,16 +608,11 @@ checkInstContents <- function(pkgdir, checkingDir)
 {
     instdocdir <- file.path(pkgdir, "inst", "doc")
     instdocdircontents <- getVigSources(instdocdir)
-    if (length(instdocdircontents))
-    {
-        if (checkingDir)
-        {
-            handleWarning(
-                "Remove vignette sources from inst/doc; ",
-                "they belong in vignettes/."
-            )
-        }
-    }
+    if (length(instdocdircontents) && checkingDir)
+        handleWarning(
+            "Remove vignette sources from inst/doc; ",
+            "they belong in vignettes/."
+        )
 }
 
 checkVigFiles <- function(vigdir, vigdircontents){
@@ -1510,45 +1515,42 @@ checkForDirectSlotAccess <- function(parsedCode, package_name)
 
 checkFunctionLengths <- function(parsedCode, pkgname)
 {
-    dflist <- vector("list", length(names(parsedCode)))
-    for (filename in names(parsedCode))
-    {
+    parsedCode <- parsedCode[grepl("\\.[Rr]$", names(parsedCode))]
+    if (!length(parsedCode))
+        return(invisible())
+    fileNames <- vapply(names(parsedCode), getDirFile, character(1L))
+    dflist <- structure(
+        vector("list", length(names(parsedCode))),
+        .Names = fileNames
+    )
+    names(parsedCode) <- fileNames
+    for (filename in names(parsedCode)) {
         pc <- parsedCode[[filename]]
-        filename <- getDirFile(filename)
         res <- getFunctionLengths(pc)
         functionNames <- names(res)
         mt <- do.call(rbind, res)
+        fname <- if (is.null(functionNames)) character(0L) else filename
         df <- cbind.data.frame(
-            filename =
-                if (is.null(functionNames)) {character(0)} else {filename},
-            functionName = functionNames, mt, row.names = NULL
+            filename = fname, functionName = functionNames, mt,
+            row.names = NULL
         )
         dflist[[filename]] <- df
     }
-    dflist <- Filter(length, dflist)
+    dflist <- Filter(nrow, dflist)
     df <- do.call(rbind, dflist)
-    colnames <- c("filename","functionName","length","startLine","endLine")
-    if (!is.null(df) && ncol(df) == length(colnames))
-    {
-        colnames(df) <- colnames
-        df <- df[with(df, order(-length)),]
-        h <- df[df$length > 50,]
-        if (nrow(h))
-        {
-            h5 <- head(df, n=5)
-            fn_msg <- apply(h5, 1L, function(row) {
+    if (length(df) && nrow(df)) {
+        df <- df[order(-df[["length"]]),]
+        h <- df[df[["length"]] > 50,]
+        if (nrow(h)) {
+            fn_msg <- apply(head(h, n=5), 1L, function(row) {
                 sprintf(
                     "%s() (%s): %s lines",
                     row['functionName'], row['filename'], row['length']
                 )
             })
-            statusmsg <- paste(
-                "There are", nrow(df), "functions greater than 50 lines."
-            )
-            statusmsg <- .singularize(df, statusmsg)
             handleNote(
                 "The recommended function length is 50 lines or less. ",
-                statusmsg,
+                .nline_report(h),
                 help_text = "The longest 5 functions are:" ,
                 messages = fn_msg
             )
@@ -1556,13 +1558,15 @@ checkFunctionLengths <- function(parsedCode, pkgname)
     }
 }
 
-.singularize <- function(obj, message) {
-    thelen <- nrow(obj)
-    if (identical(thelen, 1L)) {
-        message <- gsub("are", "is", message)
-        message <- gsub("functions", "function", message)
-    }
-    message
+.nline_report <- function(data) {
+    fnoun <- "functions"
+    plural <- !identical(nrow(data), 1L)
+    mverb <- if (plural) "are" else "is"
+    if (!plural)
+        fnoun <- substr(fnoun, 1, nchar(fnoun) - 1)
+    paste(
+        "There", mverb, nrow(data), fnoun, "greater than 50 lines."
+    )
 }
 
 checkManDocumentation <- function(package_dir, package_name, libloc)
@@ -1589,7 +1593,7 @@ checkForPromptComments <- function(pkgdir)
     bad <- vapply(manpages,
         function(manpage) {
             lines <- readLines(manpage, warn=FALSE)
-            any(grepl("^%% ~", lines))
+            any(grepl("^%%\\s+~", lines))
         },
         logical(1L)
     )
@@ -1601,41 +1605,46 @@ checkForPromptComments <- function(pkgdir)
         )
 }
 
+.tagListExtract <- function(rd, tags, Tag) {
+    if (missing(tags))
+        tags <- tools:::RdTags(rd)
+    if (!Tag %in% tags)
+        character(0L)
+    else
+        unlist(rd[tags == Tag], recursive = FALSE)
+}
+
+.tagsExtract <- function(rd, tags, Tag) {
+    tagList <- .tagListExtract(rd = rd, tags = tags, Tag = Tag)
+    as.character(tagList)
+}
+
+.valueInManPage <- function(manpage) {
+    rd <- tools::parse_Rd(manpage)
+    tags <- tools:::RdTags(rd)
+    if (identical(docType(rd, tags), "package"))
+        return(TRUE)
+    tagList <- .tagListExtract(rd, tags, "\\value")
+    values <- Filter(function(x) attr(x, "Rd_tag") != "COMMENT", tagList)
+    value <- paste(values, collapse = "")
+    nzchar(trimws(value)) && length(values)
+}
+
 checkForValueSection <- function(pkgdir)
 {
     pkgname <- basename(pkgdir)
-    manpages <- dir(file.path(pkgdir, "man"),
-        pattern="\\.Rd$", ignore.case=TRUE, full.names=TRUE)
-    ok <- vapply(manpages, function(manpage) {
-        rd <- tools::parse_Rd(manpage)
-        tags <- tools:::RdTags(rd)
-
-        type <- docType(rd)
-        if (identical(type, "data"))
-            return(TRUE)
-
-        value <- NULL
-        if ("\\usage" %in% tags && (!"\\value" %in% tags))
-            return(FALSE)
-
-        if ("\\value" %in% tags)
-            value <- rd[grep("\\value", tags)]
-
-        if ("\\usage" %in% tags && !is.null(value))
-        {
-            values <- paste(unlist(value), collapse='')
-            tst <- (is.list(value[[1]]) && length(value[[1]]) == 0) ||
-                nchar(gsub("^\\s+|\\s+$", "", values)) == 0
-            if (tst)
-                return(FALSE)
-        }
-        TRUE
-    }, logical(1))
+    manpages <- list.files(
+        path = file.path(pkgdir, "man"),
+        pattern = "\\.[Rr][Dd]$",
+        full.names = TRUE
+    )
+    ok <- vapply(manpages, .valueInManPage, logical(1))
     if (!all(ok)) {
         not_oks <- vapply(manpages[!ok], getDirFile, character(1L))
-        handleWarning(
-            "Add non-empty \\value sections to the following man pages: ",
-            messages = paste(not_oks, collapse=", "))
+        handleWarningFiles(
+            "Empty or missing \\value sections found in man pages.",
+            messages = paste(not_oks, collapse=", ")
+        )
     }
 }
 
@@ -1856,9 +1865,9 @@ checkSkipOnBioc <- function(pkgdir)
 .rmYAMLfm <- function(lines) {
     fm_idx <- grep("^---\\s*$", lines)
     if (length(fm_idx) && !identical(length(fm_idx), 2L))
-        stop("More than 2 YAML front matter delimiters, i.e., '---' found")
+        warning("More than 2 YAML front matter delimiters, i.e., '---' found")
     if (length(fm_idx))
-        lines <- lines[-seq(fm_idx[1], fm_idx[2])]
+        lines <- lines[-seq(min(fm_idx), max(fm_idx))]
     lines
 }
 
@@ -1958,47 +1967,54 @@ checkFormatting <- function(pkgdir, nlines=6)
     }
 }
 
-checkIsPackageNameAlreadyInUse <- function(pkgName,
-    repo = c("CRAN", "BioCsoft", "BioCann",
-        "BioCexp", "BioCworkflows", "BioCbooks")
+checkIsPackageNameAlreadyInUse <- function(
+    pkgName,
+    repo = c(
+        "CRAN", "BioCsoft", "BioCann", "BioCexp", "BioCworkflows", "BioCbooks"
+    )
 ) {
     repo <- match.arg(repo)
+    msg <- paste0("'", pkgName, "' already exists in Bioconductor.")
 
-    if (identical(repo, "CRAN")) {
-        repo.url <- sprintf(
+    if (identical(repo, "CRAN"))
+        msg <- paste(
+            "The package already exists on CRAN. Packages submitted to",
+            "Bioconductor must be removed from CRAN before the next",
+            "Bioconductor release."
+        )
+    
+    repo.url <- switch(
+        repo,
+        CRAN = sprintf(
             "%s/src/contrib/PACKAGES", BiocManager::repositories()[repo]
-        )
-    } else {
-        repo.url <- switch(
-            repo,
-            BiocSoft = "http://bioconductor.org/packages/devel/bioc/VIEWS",
-            BioCann =
-                "http://bioconductor.org/packages/devel/data/annotation/VIEWS",
-            BioCexp =
-                "http://bioconductor.org/packages/devel/data/experiment/VIEWS",
-            BioCworkflows =
-                "http://bioconductor.org/packages/devel/workflows/VIEWS",
-            BioCbooks =
-                "https://bioconductor.org/packages/devel/books/VIEWS",
-            "http://bioconductor.org/packages/devel/bioc/VIEWS"
-        )
-    }
+        ),
+        BioCsoft = "http://bioconductor.org/packages/devel/bioc/VIEWS",
+        BioCann =
+            "http://bioconductor.org/packages/devel/data/annotation/VIEWS",
+        BioCexp =
+            "http://bioconductor.org/packages/devel/data/experiment/VIEWS",
+        BioCworkflows =
+            "http://bioconductor.org/packages/devel/workflows/VIEWS",
+        BioCbooks =
+            "https://bioconductor.org/packages/devel/books/VIEWS"
+    )
 
     conn <- url(repo.url)
-    dcf <- tryCatch(suppressWarnings(read.dcf(conn)), error=identity)
-    close(conn)
-    if (inherits(dcf, "error")) {
-        handleMessage(
-            "Unable to access repository ", BiocManager::repositories()[repo]
+    on.exit({ close(conn) })
+    dcf <- try(read.dcf(conn), silent = TRUE)
+    if (is(dcf, "try-error"))
+        return(
+            handleMessage(
+                "Unable to access repository ",
+                BiocManager::repositories()[repo]
+            )
         )
-    } else if (tolower(pkgName) %in% tolower(dcf[,"Package"])) {
-        if (repo == "CRAN")
-            msg <- "Package must be removed from CRAN."
-        else {
-            msg <- paste0("'", pkgName, "' already exists in Bioconductor.")
-        }
+        
+    isDuplicate <- tolower(pkgName) %in% tolower(dcf[,"Package"])
+    if (isDuplicate && identical(repo, "CRAN"))
+        handleWarning(msg)
+    else if (isDuplicate)
         handleError(msg)
-    }
 }
 
 checkForBiocDevelSubscription <- function(pkgdir)
